class Object : public AR_Sprite
 {
  protected:

   Bullet **bullets;                  // Array of bullets (for collision)
   Cannon **cannons;                  // Array of cannons
   Shield *shield;                    // Shield
   AR_Light  **lights;                // Array of lights
   AR_Sample **samples;               // Array of AR_Sample for sound effects
   AR_ParticleGenerator **generators; // Array of particle generators
   int total_cannons;
   int total_generators;
   int total_lights;
   int health;

   void reset()
    {
	 // Reset variables to default
	 AR_Sprite::reset();
	 setCenterX(width>>1); setCenterY(height>>1);
	 health = 0;
	 enable = false; // All object are disabled for default
	 for(int i=0;i<total_generators;i++) generators[i]->reset();
	 for(int i=0;i<total_cannons;i++)    cannons[i]->reset();
	 shield->reset();
	}

  public:

   Object(string filename, AR_Display *display, AR_Sprite *p_particles, AR_Sprite *p_cannons, AR_Sprite *p_shield, AR_Sprite *p_lights, AR_Sprite *p = NULL, int total_cannons = 2, int total_generators = 3, int collision_mask_type = AR::DYNAMIC_COLLISION_MASK) : AR_Sprite(filename,display,0,0,p,collision_mask_type)
    {
	 this->total_cannons    = total_cannons;
	 this->total_generators = total_generators;
     this->total_lights     = 1;//total_lights;

	 generators = (AR_ParticleGenerator **)malloc(sizeof(AR_ParticleGenerator *)*total_generators);
	 for(int i=0;i<total_generators;i++) generators[i] = new AR_ParticleGenerator("",display,50,p_particles);

     cannons = (Cannon **)malloc(sizeof(Cannon *)*total_cannons);
	 for(int i=0;i<total_cannons;i++) cannons[i] = new Cannon("",display,100,p_cannons);

     lights = (AR_Light **)malloc(sizeof(AR_Light *)*total_lights);
	 for(int i=0;i<total_lights;i++) lights[i] = new AR_Light("",display,p_lights);

	 shield = new Shield("",display,(Shield *)p_shield);

	 reset();
	}

   ~Object() { }

   void run()
    {
     if(!enable) return;

     AR_Sprite::run();
     for(int i=0;i<total_generators;i++) generators[i]->run();
     for(int i=0;i<total_cannons;   i++) cannons[i]   ->run();
     for(int i=0;i<total_lights;    i++) lights[i]    ->run();
     shield->run();
	}

   void draw()
    {
	 AR_Sprite::draw();
	}

   void hit(Bullet *b)
    {
	 health -= b->getDamage();
	 if(health < 0) health = 0;
	}

   void drawHealthbar()
    {
	 float scroll_x,scroll_y;

	 if(!enable) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;
     AR_Util::rectangle(x-scroll_x-center_x,y-scroll_y-center_y-6,health,4,0,200,0);
	}

   void drawShieldsbar()
    {
	 float scroll_x,scroll_y;

	 if(!enable) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;
     AR_Util::rectangle(x-scroll_x-center_x,y-scroll_y-center_y-12,getShieldHealth(),4,0,50,230);
	}

   void setScrollXYP(float *scroll_x, float * scroll_y)
    {
	 setScrollXP(scroll_x); setScrollYP(scroll_y);
	 for(int i=0;i<total_generators;i++) generators[i]->setScrollXYP(scroll_x,scroll_y);
	 for(int i=0;i<total_cannons;   i++) cannons[i]   ->setScrollXYP(scroll_x,scroll_y);
	 for(int i=0;i<total_lights;    i++) lights[i]    ->setScrollXYP(scroll_x,scroll_y);
	 if(shield) shield->setScrollXYP(scroll_x,scroll_y);
	}

   void setHealth(int health) { this->health = health; }
   
   void setSamplesP(AR_Sample **samples) { this->samples = samples; }

   int getHealth()             { return health;                 }

   int getTotalCannons()       { return total_cannons;          }

   int getTotalGenerators()    { return total_generators;       }

   float getCannonAngle(int n) { return cannons[n]->getAngle();        }

   int getTotalBullets(int n)  { return cannons[n]->getTotalBullets(); }
   
   int getShieldHealth()       { return shield->getHealth();           }

   float getScrollX()          { return *scroll_x;                     }
   
   float getScrollY()          { return *scroll_y;                     }

   Cannon *getCannon(int n)    { return cannons[n];                    }

   Shield *getShield()         { return shield;                        }
   
   Bullet **getBulletsP(int n) { return cannons[n]->getBulletsP();     }

 };
